@@ -1,13 +1,8 @@
 use crate::raw::{
-    capacity::{capacity, Round},
-    Storage, StorageWithCapacity,
+    AllocError, AllocResult, Storage, StorageWithCapacity,
 };
 
-use core::{
-    alloc::Layout,
-    mem::{align_of, size_of},
-    ptr::NonNull,
-};
+use core::{alloc::Layout, ptr::NonNull};
 use std::{alloc::handle_alloc_error, mem::MaybeUninit};
 
 use std::alloc::{Allocator, Global};
@@ -88,28 +83,28 @@ impl<T, A: Allocator + Default> Default for Heap<T, A> {
     fn default() -> Self { Self::with_alloc(Default::default()) }
 }
 
-unsafe impl<T, U, A: Allocator> Storage<U> for Heap<T, A> {
-    const IS_ALIGNED: bool = align_of::<T>() >= align_of::<U>();
+impl<T, A: Allocator> AsRef<[MaybeUninit<T>]> for Heap<T, A> {
+    fn as_ref(&self) -> &[MaybeUninit<T>] { self.0.as_ref() }
+}
 
-    fn capacity(&self) -> usize { capacity(self.0.len(), size_of::<T>(), size_of::<U>(), Round::Down) }
+impl<T, A: Allocator> AsMut<[MaybeUninit<T>]> for Heap<T, A> {
+    fn as_mut(&mut self) -> &mut [MaybeUninit<T>] { self.0.as_mut() }
+}
 
-    fn as_ptr(&self) -> *const U { (self.0.as_ptr() as *const T).cast() }
-
-    fn as_mut_ptr(&mut self) -> *mut U { (self.0.as_mut_ptr() as *mut T).cast() }
+unsafe impl<T, A: Allocator> Storage for Heap<T, A> {
+    type Item = T;
 
     fn reserve(&mut self, new_capacity: usize) {
-        let new_capacity = capacity(new_capacity, size_of::<U>(), size_of::<T>(), Round::Up);
         if self.0.len() < new_capacity {
             let _ = self.reserve_slow(new_capacity, OnFailure::Abort);
         }
     }
 
-    fn try_reserve(&mut self, new_capacity: usize) -> bool {
-        let new_capacity = capacity(new_capacity, size_of::<U>(), size_of::<T>(), Round::Up);
+    fn try_reserve(&mut self, new_capacity: usize) -> AllocResult {
         if self.0.len() < new_capacity {
             self.reserve_slow(new_capacity, OnFailure::Error)
         } else {
-            true
+            Ok(())
         }
     }
 }
@@ -118,16 +113,16 @@ impl<T, A: Default + Allocator> Heap<T, A> {
     fn with_capacity(capacity: usize) -> Self { Self(Box::new_uninit_slice_in(capacity, A::default())) }
 }
 
-unsafe impl<T, U, A: Default + Allocator> StorageWithCapacity<U> for Heap<T, A> {
+unsafe impl<T, A: Default + Allocator> StorageWithCapacity for Heap<T, A> {
     fn with_capacity(cap: usize) -> Self {
-        Self::with_capacity(capacity(cap, size_of::<U>(), size_of::<T>(), Round::Up))
+        Self::with_capacity(cap)
     }
 }
 
 impl<T, A: Allocator> Heap<T, A> {
     #[cold]
     #[inline(never)]
-    fn reserve_slow(&mut self, new_capacity: usize, on_failure: OnFailure) -> bool {
+    fn reserve_slow(&mut self, new_capacity: usize, on_failure: OnFailure) -> AllocResult {
         assert!(new_capacity > self.0.len());
 
         // taking a copy of the box so we can get it's contents and then update it later
@@ -153,7 +148,7 @@ impl<T, A: Allocator> Heap<T, A> {
         let ptr = match (ptr, on_failure) {
             (Ok(ptr), _) => ptr,
             (Err(_), OnFailure::Abort) => handle_alloc_error(layout),
-            (Err(_), OnFailure::Error) => return false,
+            (Err(_), OnFailure::Error) => return Err(AllocError),
         };
 
         // Creating a new Heap using the re-alloced pointer.
@@ -165,6 +160,6 @@ impl<T, A: Allocator> Heap<T, A> {
             std::mem::forget(old);
         }
 
-        true
+        Ok(())
     }
 }
